#ifndef INCLUDED_CURVE_H
#define INCLUDED_CURVE_H

#pragma warning(disable : 4786)

#include <vector>

#include <iostream>

#include <string>

#include "Point.h"

#include "Color.h"

#include "Zoomer.h"

using namespace std;

class Curve
{
	public:

		Curve(void);

		Curve(const string & name, const double & start_value = 0);

		void performNewCurveSetup(const double & start_value = 0);

		Curve(istream & input_stream);

		void toStream(ostream & output_stream) const;

		void fromStream(istream & input_stream);

		string getName(void) const;

		void setCurveType(const string & curve_type);
		
		double evaluateCurveAt(const double & x) const;

		double getMinimumX(void) const;

		double getMaximumX(void) const;

		void scaleX(const double & scaling_factor);

		void addControlPoint(const Point & normalized_point);

		void removeControlPoint(const unsigned int & control_point_index);

		unsigned int getClosestControlPoint(const Point & point, double * closest_distance) const;

		double getDistanceToCurve(const Point & normalized_point) const;

		Point getControlPoint(const unsigned int & control_point_index) const;

		void changeControlPoint(const unsigned int & control_point_index, const Point & normalized_new_point);

		unsigned int getNumberOfControlPoints(void) const;
		unsigned int getNumberOfEvaluatedCurveSegments(void) const;

		bool isShown(void) const;

		bool toggleShowCurve(void);

		void setWrapControlPointsFlag(const bool & wrap_control_points);

		void drawEvaluatedCurveSegments(void) const;

		void drawControlPoints(void) const;

		void drawCurve(void) const;

		Zoomer * getZoomer(void);

		const Zoomer * getZoomer(void) const;

		void markDirty(void) const;

	private:

		void fixIfDirty(void) const;

		mutable vector<Point>	m_control_points;

		string					m_name;

		string					m_current_curve_type;

		bool						m_currently_shown;

		bool						m_wrap_control_points;

		Color						m_color;

		Zoomer						m_zoom;

		mutable bool				m_dirty;

		mutable vector<Point>	m_evaluated_curve_points;
};

ostream & operator<<(ostream & output_stream, const Curve & curve_data);
istream & operator>>(istream & input_stream, Curve & curve_data);

#endif