//
// fltkCurveEditor.h
//
// The header file for the CurveEditor
//

#ifndef __fltkCurveEditor_h__
#define __fltkCurveEditor_h__

#pragma warning(disable:4786)

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Float_Input.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Group.H>
#include <FL/fl_draw.H>

#include <stdlib.h>

#include "CurveEditor.h"

class CAniUI;
class Scene;
class curveDisplay;
class Object;

extern CurveEditor *curve_editor;

class textBox : public Fl_Widget {
public:
  textBox(int X,int Y,int W,int H) : Fl_Widget(X,Y,W,H) {}
  void draw() {
    fl_color(FL_GRAY);
    fl_rectf(x(),y(),w(),h());
  }
};

class fltkCurveEditor  {
public:
	fltkCurveEditor(int x, int y, int w, int h, const char *l);

	// redraw curveDisplay
	void refresh();
	// redraw all widgets
	void redraw();

	void show();
	void hide();

	void reset() {
		curve_editor->resetCurveEditor();
		if (m_pWrapCurveButtom->value()) {
			m_pWrapCurveButtom->clear();
			curve_editor->toggleWrapControlPoints();
		}
	}

	void setUI(CAniUI* ui) { m_pUI=ui; };
	CAniUI* getUI() { return m_pUI; };

	void setScene(Scene *scn);

	float getMaxAniTimeInput() { return (float)atof(m_pMaxAniTimeInput->value()); };
	float getScaleFactorInput() { return (float)atof(m_pScaleFactorInput->value()); };
	float getMaxValueInput() { return (float)atof(m_pMaxValueInput->value()); };
	float getMinValueInput() { return (float)atof(m_pMinValueInput->value()); };

	int getStartTime();
	int getEndTime();

	void resetCurveMenu(Object *obj);

	void updateCurrentlySelectedCurve(const char *name);
	void updateMaxMin(float max, float min);

	void updateTimeScale();
	void updateValueScale();

private:
	int m_nWindowWidth, m_nWindowHeight;

	Fl_Window *m_pWindow;
	
	Fl_Button *m_pUndoZoomButtom;
	Fl_Check_Button *m_pWrapCurveButtom;

	textBox *m_pTextContainerBox;
	Fl_Box *m_pSelectCurveText;
	Fl_Box *m_pNumCtrlPointsText;
	Fl_Box *m_pNumCtrlSegmentsText;
	Fl_Box *m_pFPSText;

	Fl_Float_Input *m_pMaxAniTimeInput;
	Fl_Float_Input *m_pScaleFactorInput;
	Fl_Float_Input *m_pMaxValueInput;
	Fl_Float_Input *m_pMinValueInput;

	Fl_Button *m_pMaxAniTimeApplyButton;
	Fl_Button *m_pScaleFactorApplyButton;
	Fl_Button *m_pMaxValueApplyButton;
	Fl_Button *m_pMinValueApplyButton;

	Fl_Int_Input *m_pStartInput;
	Fl_Int_Input *m_pEndInput;
	Fl_Button *m_pSetStartEndButton;

	Fl_Box *m_pTimeScale[32];
	Fl_Box *m_pValueScale[32];

	curveDisplay *m_pCurveDisplay;

	char m_sSelectCurveText[64];
	char m_sNumCtrlPointsText[8];
	char m_sNumCtrlSegmentsText[8];
	char m_sFPSText[8];

	Scene *m_pScene;

	CAniUI *m_pUI;

	int m_nStart;
	int m_nEnd;
	float m_fAniLength;

	// static callback functions
//	static void cb_exit(Fl_Widget* o, void* v);
	static void cb_max_ani_time(Fl_Widget* o, void* v);
	static void cb_scale_factor(Fl_Widget* o, void* v);
	static void cb_max_min_value(Fl_Widget* o, void* v);
	static void cb_set_start_end(Fl_Widget* o, void* v);
	static void cb_wrap_curve(Fl_Widget* o, void* v);
	static void cb_undo_zoom(Fl_Widget* o, void* v);
};

#endif