//
// glView.cpp
//
// The code maintaining the view for the rendered image
//

#include "glView.h"
#include "aniUI.h"
#include "bitmap.h"


CGLView::CGLView(int x, int y, int w, int h, const char *l, Scene* scn)
			: Fl_Gl_Window(x,y,w,h,l)
{
	m_nWindowWidth = w;
	m_nWindowHeight = h;

	m_pBuf=NULL;
	m_bSizeChange=true;

	m_pScene=scn;

	user_data((void*)(this));

	callback(cb_exit);
	when(FL_HIDE);
}

void CGLView::cb_exit(Fl_Widget* o, void* v) 
{	
	CGLView* self=(CGLView *)(o->user_data());

	if (self->m_pScene) delete self->m_pScene;

	((CAniUI*)(self->getUI()))->removeView(self);
	self->hide();
}

void CGLView::setScene(Scene *scn)
{
	m_pScene=scn;
}

int CGLView::handle(int event)
{
	return 1;
}

void CGLView::draw()
{
	if(!valid())
	{
		m_nWindowWidth=w();
		m_nWindowHeight=h();

		m_bSizeChange=true;

		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		gluPerspective( 30.0, (double)m_nWindowWidth/m_nWindowHeight, 1.0, 100.0 );
		glViewport( 0, 0, m_nWindowWidth, m_nWindowHeight );

		glEnable(GL_DEPTH_TEST);

		glClearColor(0.0f, 0.0f, 0.0f, 0.0);
	}

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// get drawMode and drawQuality from UI
	int drawMode=m_pUI->getDrawMode();

	if (m_pScene)
		m_pScene->redraw(drawMode);

}

void CGLView::refresh()
{
	redraw();
}

void CGLView::resizeWindow(int width, int height)
{
	resize(x(), y(), width, height);
	m_nWindowWidth=w();
	m_nWindowHeight=h();
}

void CGLView::saveImage(char *iname)
{
	if (m_bSizeChange) {
		if (m_pBuf) delete [] m_pBuf;
		m_pBuf=new unsigned char [m_nWindowWidth*m_nWindowHeight*3];
		m_bSizeChange=false;
	}

	make_current();

	glReadBuffer(GL_BACK);

	glPixelStorei( GL_PACK_ALIGNMENT, 1 );
	glPixelStorei( GL_PACK_ROW_LENGTH, m_nWindowWidth );
	
	glReadPixels( 0, 0, m_nWindowWidth, m_nWindowHeight, GL_RGB, GL_UNSIGNED_BYTE, m_pBuf );

	writeBMP(iname, m_nWindowWidth, m_nWindowHeight, m_pBuf); 
}
